
from __future__ import absolute_import, division, print_function
from builtins import (bytes, str, open, super, range,
  zip, round, input, int, pow, object)

from mrkv import Markov
import unittest
import random
from collections import Counter

class MarkovTester(unittest.TestCase):
  def test_deterministic_chain(self):
    # should generate a specific sequence 
    states = ['a', 'b', 'c', 'd', 'e']
    m = Markov(order=2)
    m.addTransitions(states)
    seq = m.generateSequence(['a','b'], 3)
    self.assertTrue(seq == ['c', 'd', 'e'])

  def test_probabilistic_generation(self):
    # should generate approximately 
    # equal numbers of 'a's and 'b's
    m = Markov(order=1)
    m.addTransition(tuple('a'), 'a')
    m.addTransition(tuple('a'), 'b')
    m.addTransition(tuple('b'), 'a')
    m.addTransition(tuple('b'), 'b')
    count = 100000
    epsilon = 1000
    seq = m.generateSequence('a', count)
    c = Counter(seq)
    self.assertTrue(c['a'] + c['b'] == count)
    self.assertTrue(abs(c['a'] - c['b']) < epsilon)

if __name__ == '__main__':
  unittest.main(verbosity=2)





from __future__ import absolute_import, division, print_function
from builtins import (bytes, str, open, super, range,
  zip, round, input, int, pow, object)

from collections import defaultdict, Counter
import random

class MarkovError(Exception):
  pass

class Markov(object):
  def __init__(self, order=1): 
    self._order = order
    self._rules = defaultdict(Counter)
    self.default = None

  def addTransition(self, pred, succ):
    '''Add/update a transition by passing a predecessor and a successor.'''
    if len(pred) != self._order:
      raise MarkovError('Predecessor is not of correct order.')
    self._rules[pred].update([succ])

  def generate(self, pred):
    '''Generate a single state given a predecessor.'''
    choice = self.default

    elts = self._rules[pred].most_common()
    totalSum = 0
    for _, count in elts:
      totalSum += count

    rnd = random.random()
    currentSum = 0
    for elt, count in elts:
      if rnd >= currentSum / totalSum \
          and rnd <= (currentSum + count) / totalSum:
        choice = elt
      currentSum += count
        
    return choice

  def addTransitions(self, states):
    '''Add a set of transitions given a sequence of states.''' 
    limit = len(states) - self._order
    if limit < 0:
      raise MarkovError('State list is too short.')

    for i in range(limit):
      pred = []
      for j in range(self._order):
        pred.append(states[i+j])
      succ = states[i + self._order]
      self.addTransition(tuple(pred), succ)
    
  def generateSequence(self, pred, length):
    '''Generate a sequence of desired length given a predecessor.'''
    seq = list(pred)
    for i in range(length):
      succ = self.generate(tuple(seq[i:i+self._order]))
      seq.append(succ)
    return seq[self._order:]




from setuptools import setup

setup(name='mrkv',
  version='0.0.1',
  description='Tiny Markov chain library',
  url='http://github.com/erdiaker/mrkv',
  author='Erdi Aker',
  author_email='erdiaker@gmail.com',
  license='MIT',
  py_modules=['mrkv'],
  install_requires=['future'],
  zip_safe=False)
